<?php
// $Id: govtrack.inc,v 1.35 2009/04/15 08:26:51 drumm Exp $

function legislature_govtrack_politicians() {
  include_once drupal_get_path('module', 'legislature') .'/xml.inc';

  set_time_limit(0);
  $session_years = legislature_session_years();
  $start_years = array();
  foreach (legislature_get_enabled_sessions('us') as $session) {
    $start_years[] = $session_years['us'][$session]['start'];
  }

  $result = drupal_http_request('http://www.govtrack.us/data/us/111/repstats/people.xml');
  if (!legislature_xml_parse($result->data, 'legislature_govtrack_politicians_xml', min($start_years))) {
    watchdog('legislature', t('Failed to parse people.xml. @error', array('@error' => $result->error)), WATCHDOG_ERROR);
  }
}

function legislature_govtrack_politicians_xml($op, $parser, $data, $attributes = NULL) {
  static $genders = array(
    'M' => 'Male',
    'F' => 'Female',
  );
  static $offices = array(
    'rep' => 'House',
    'sen' => 'Senate',
  );
  static $start_year;
  static $politician;
  static $elected_jobs = array();

  switch ($op) {
    case 'setup':
      $start_year = gmmktime(0, 0, 0, 1, 1, $data);
      break;

    case 'start':
      switch ($data) {
        case 'PERSON':
          $politician = array(
            'last_name' => $attributes['LASTNAME'],
            'full_name' => $attributes['FIRSTNAME'] .' '. $attributes['LASTNAME'],
            'middle_name' => $attributes['MIDDLENAME'],
            'nick_name' => $attributes['NICKNAME'],
            'first_name' => $attributes['FIRSTNAME'],
            'name_modifier' => $attributes['NAMEMOD'],
            'roster_name' => $attributes['NAME'],
            'gender' => $genders[$attributes['GENDER']],
            'govtrack_id' => $attributes['ID'],
            'crp_id' => $attributes['OSID'],
          );
          break;

        case 'ROLE':
          $elected_job = array(
            'district_id' => legislature_district('us', $offices[$attributes['TYPE']], $attributes['STATE'], $attributes['DISTRICT'], $attributes['STATE'] .' '. $offices[$attributes['TYPE']] .' District '. $attributes['DISTRICT']),
            'office_id' => legislature_get_office_id('US', $offices[$attributes['TYPE']]),
            'is_candidate' => FALSE,
          );
          list($year, $month, $day) = explode('-', $attributes['STARTDATE']);
          $elected_job['start'] = mktime(0, 0, 0, $month, $day, $year, 0);
          list($year, $month, $day) = explode('-', $attributes['ENDDATE']);
          $elected_job['end'] = mktime(0, 0, 0, $month, $day, $year, 0);

          $elected_jobs[] = $elected_job;

          if ($elected_job['end'] > $start_year) {
            // Trust party data after the start year, sometimes people switch.
            // And the party being set decides if we actually import someone or
            // not.
            $politician['party'] = $attributes['PARTY'];
          }
          break;
      }
      break;

    case 'end':
      switch ($data) {
        case 'PERSON':
          if (isset($politician['party'])) {
            $politician_id = legislature_politician($politician, 'govtrack');
            if (!is_null($politician_id)) {
              legislature_save_elected_jobs('us', $politician_id, $elected_jobs);
            }
          }
          $politician = NULL;
          $elected_jobs = array();
          break;
      }
      break;
  }
}

function legislature_govtrack_bills($clear = array()) {
  set_time_limit(0);

  foreach ($clear as $session => $clear_session) {
    if ($clear_session) {
      db_query("UPDATE {legislature_bill} b INNER JOIN {legislature_session} s ON s.session_id = b.session_id AND s.session_jurisdiction = 'us' AND s.session_name = '%s' SET last_version = ''", $session);
    }
  }

  foreach (legislature_get_enabled_sessions('us') as $congress) {
    legislature_govtrack_bills_start(NULL, 'setup', $congress);
    $result = drupal_http_request('http://www.govtrack.us/data/us/'. $congress .'/bills.index.xml');
    $parser = drupal_xml_parser_create($result->data);
    xml_set_element_handler($parser, 'legislature_govtrack_bills_start', 'legislature_govtrack_bills_end');
    xml_set_character_data_handler($parser, 'legislature_govtrack_bills_data');
    if (!xml_parse($parser, $result->data, 1)) {
      watchdog('legislature', t('Failed to parse bills.xml. @error', array('@error' => $result->error)));
    }
    xml_parser_free($parser);
  }
}

function legislature_govtrack_bills_start($parser, $name, $attributes) {
  static $session_id;

  switch ($name) {
    case 'setup':
      $session_id = db_result(db_query("SELECT session_id FROM {legislature_session} WHERE session_jurisdiction = 'us' AND session_name = '%s'", $attributes));
      break;

    case 'BILL':
      list($dummy, $title) = explode(': ', $attributes['TITLE'], 2);
      $bill = array(
        'session_id' => $session_id,
        'prefix' => drupal_strtoupper($attributes['TYPE']),
        'measure' => drupal_strtoupper($attributes['TYPE']) .' '. $attributes['NUMBER'],
        'topic' => $title,
        'description' => $attributes['OFFICIAL-TITLE'],
        'status' => $attributes['STATUS'],
      );
      if (isset($attributes['LAST-ACTION'])) {
        $bill['last_version'] = $attributes['LAST-ACTION'];
      }
      else {
        $bill['last_version'] = $attributes['LASTACTION'];
      }
      if (legislature_save_bill($bill)) {
        job_queue_add('legislature_govtrack_update_bill', t('Download bill %measure', array('%measure' => $bill['measure'])), array($bill['bill_id']), drupal_get_path('module', 'legislature') .'/imports/us/govtrack.inc');
      }
      break;
  }
}

function legislature_govtrack_bills_end($parser, $name) {
}

function legislature_govtrack_bills_data($parser, $data) {
}

/**
 * Update a single bill.
 */
function legislature_govtrack_update_bill($bill_id) {
  include_once drupal_get_path('module', 'legislature') .'/xml.inc';

  $result = db_query('SELECT b.bill_id, s.session_name, b.session_id, b.measure FROM {legislature_bill} b INNER JOIN {legislature_session} s ON s.session_id = b.session_id WHERE b.bill_id = %d', $bill_id);
  if (db_num_rows($result) > 0) {
    $bill = db_fetch_object($result);
    $url = 'http://www.govtrack.us/data/us/'. $bill->session_name .'/bills/'. implode('', explode(' ', drupal_strtolower($bill->measure))) .'.xml';
    $result = drupal_http_request($url);
    legislature_govtrack_update_bill_xml('setup', NULL, $bill);
    if (!legislature_xml_parse($result->data, 'legislature_govtrack_update_bill_xml')) {
      watchdog('legislature', t('Failed to update bill %id. (%url) %error', array('%id' => $bill_id, '%url' => $url, '%error' => $result->error)));
      job_queue_add('legislature_govtrack_update_bill', t('Download bill %id', array('%id' => $bill_id)), array($bill->bill_id), drupal_get_path('module', 'legislature') .'/imports/us/govtrack.inc');
    }
    else {
      module_invoke_all('legislature', 'bill', 'update ran', array('bill_id' => $bill_id));
    }

    // Get roll votes
    $url = 'http://www.govtrack.us/congress/votes_download_xml.xpd?bill='. implode($bill->session_name .'-', explode(' ', drupal_strtolower($bill->measure)));
    $result = drupal_http_request($url);
    if (!legislature_xml_parse($result->data, 'legislature_govtrack_update_bill_votes_xml', $bill)) {
      watchdog('legislature', t('Failed to update vote for bill %id. (%url) %error', array('%id' => $bill_id, '%url' => $url, '%error' => $result->error)));
      job_queue_add('legislature_govtrack_update_bill', t('Download bill %id', array('%id' => $bill_id)), array($bill->bill_id), drupal_get_path('module', 'legislature') .'/imports/us/govtrack.inc');
    }
  }
  else {
    watchdog('legislature', t('Skipping bill updates for %bill_id because the bill is missing,', array('%bill_id' => $bill_id)));
  }
}

function legislature_govtrack_update_bill_xml($op, $parser, $data, $attributes = NULL) {
  static $bill, $content, $last_content, $last_op, $in_actions = 0, $action, $fault = FALSE, $action_ids = array();

  if ($fault && $op != 'setup') {
    return;
  }

  $houses = array(
    's' => 'Senate',
    'h' => 'House',
  );

  switch ($op) {
    case 'setup':
      $fault = FALSE;
      $bill = $data;
      db_query('DELETE FROM {legislature_bill_title} WHERE bill_id = %d', $bill->bill_id);
      db_query('DELETE FROM {legislature_bill_sponsor} WHERE bill_id = %d', $bill->bill_id);
      db_query('DELETE FROM {legislature_bill_committee} WHERE bill_id = %d', $bill->bill_id);
      break;

    case 'start':
      if ($in_actions > 0) {
        $in_actions += 1;

        if ($in_actions == 2) { // Top-level action element
          preg_match('/(\d\d\d\d)-(\d\d)-(\d\d).*/', $attributes['DATETIME'], $matches);
          list($dummy, $year, $month, $day) = $matches;

          // Votes are a special case
          if ($data == 'VOTE' && !isset($attributes['ROLL'])) {
            $action = array(
              'bill_id' => $bill->bill_id,
              'description' => 'vote',
              'date' => preg_replace('/(\d\d\d\d)-(\d\d)-(\d\d).*/', '\1\2\3', $attributes['DATETIME']),
              'date_int' => gmmktime(0, 0, 0, $month, $day, $year),
              'location' => $houses[$attributes['WHERE']],
              'outcome' => drupal_strtoupper($attributes['RESULT']),
              'is_vote' => TRUE,
              'is_substantive' => TRUE,
              'ayes' => 0,
              'noes' => 0,
            );
          }
          else {
            $action = array(
              'bill_id' => $bill->bill_id,
              'description' => drupal_strtolower($data),
              'date' => preg_replace('/(\d\d\d\d)-(\d\d)-(\d\d).*/', '\1\2\3', $attributes['DATETIME']),
              'date_int' => gmmktime(0, 0, 0, $month, $day, $year),
              'location' => '',
              'outcome' => '',
              'is_vote' => FALSE,
              'is_substantive' => FALSE,
              'ayes' => 0,
              'noes' => 0,
            );
          }

          // If we have a high-quality unix timestamp, use it.
          if (isset($action) && isset($attributes['DATE'])) {
            $action['date_int'] = $attributes['DATE'];
          }
        }
      }

      switch ($data) {
        case 'TITLE':
          $content = array(
            'type' => $attributes['TYPE'],
            'status_as' => $attributes['AS'],
          );
          break;

        case 'SPONSOR':
          db_query('INSERT INTO {legislature_bill_sponsor} (bill_id, politician_id, type) VALUES (%d, %d, %d)', $bill->bill_id, legislature_get_politician_id('govtrack', $attributes['ID']), LEGISLATURE_SPONSOR);
          break;

        case 'COSPONSOR':
          db_query('INSERT INTO {legislature_bill_sponsor} (bill_id, politician_id, type) VALUES (%d, %d, %d)', $bill->bill_id, legislature_get_politician_id('govtrack', $attributes['ID']), LEGISLATURE_COSPONSOR);
          break;

        case 'COMMITTEE':
          $committee_id = db_result(db_query("SELECT committee_id FROM {legislature_committee_name} WHERE session_id = %d AND name = '%s'", $bill->session_id, $attributes['NAME']));
          if (!empty($committee_id)) {
            db_query("INSERT INTO {legislature_bill_committee} (bill_id, committee_id, activity) VALUES (%d, %d, '%s')", $bill->bill_id, $committee_id, $attributes['ACTIVITY']);
          }
          break;

        case 'AMENDMENT':
          legislature_govtrack_amendment($bill->session_name, $bill->bill_id, $attributes['NUMBER']);
          break;

        case 'ACTIONS':
          $in_actions = 1;
          break;
      }
      break;

    case 'data':
      if ($last_op === 'data') {
        $last_content .= $data;
      }
      else {
        $last_content = $data;
      }
      break;

    case 'end':
      switch ($data) {
        case 'TITLE':
          db_query("INSERT INTO {legislature_bill_title} (bill_id, title, type, status_as) VALUES (%d, '%s', '%s', '%s')", $bill->bill_id, $last_content, $content['type'], $content['status_as']);
          break;

        case 'BILL':
          // Delete non-vote actions which we have not just imported.
          db_query("DELETE FROM {legislature_action} WHERE bill_id = %d AND action_id NOT IN (". implode(',', array_fill(0, count($action_ids), "%d")) .") AND description != 'vote'", array_merge(array($bill->bill_id), $action_ids));
          break;

        case 'ACTIONS':
          $in_actions = 0;
          break;

        case 'TEXT':
          if ($in_actions == 3 && isset($action)) {
            $action['motion'] = $last_content;
          }
      }

      if ($in_actions > 0) {
        $in_actions -= 1;
        if ($in_actions == 1 && isset($action)) {
          if (legislature_save_action($action, array('bill_id', 'description', 'date', 'motion'))) {
            db_query('DELETE FROM {legislature_action_politician} WHERE action_id = %d', $action['action_id']);

            // Vote actions imported here do not have full rolls, so we need to
            // simulate this and fill out the legislators' individual votes.
            if ($action['is_vote']) {
              db_query("INSERT INTO {legislature_action_politician} (politician_id, action_id, vote) SELECT politician_id, %d, '%s' FROM {legislature_elected_job} ej INNER JOIN {legislature_elected_job_session} ejs ON ej.elected_job_id = ejs.elected_job_id AND ejs.session_id = %d INNER JOIN {legislature_office} o ON o.office_id = ej.office_id AND o.title = '%s' INNER JOIN {legislature_district} d ON d.district_id = ej.district_id AND d.state NOT IN ('DC', 'AS', 'GU', 'VI', 'PR') WHERE ej.start < %d AND %d < ej.end", $action['action_id'], ($action['outcome'] == 'PASS') ? 'AYE' : 'NOE', $bill->session_id, $action['location'], $action['date_int'], $action['date_int']);
              if ($action['outcome'] == 'PASS') {
                $action['ayes'] = db_affected_rows();
              }
              else {
                $action['noes'] = db_affected_rows();
              }
              legislature_save_action($action);
            }

            $action_ids[] = $action['action_id'];
          }
          else {
            // There are too many actions for some reason. Delete em all and start over.
            watchdog('legislature', t('This should not happen!! Found %n actions for bill %bill_id action %description.', array('%n' => $rows, '%bill_id' => $action['bill_id'], '%description' => $action['description'])));
            $result = db_query('SELECT action_id FROM {legislature_action} WHERE bill_id = %d', $action['bill_id']);
            db_query('DELETE FROM {legislature_action} WHERE bill_id = %d', $action['bill_id']);
            $fault = TRUE;
            job_queue_add('legislature_govtrack_update_bill', t('Download bill %id (requeue fault)', array('%id' => $action['bill_id'])), array($action['bill_id']), drupal_get_path('module', 'legislature') .'/imports/us/govtrack.inc');
            return;
          }

          $action = NULL;
        }
      }
      break;
  }

  $last_op = $op;
}

function legislature_govtrack_amendment($session, $bill_id, $amendment_number) {
  include_once drupal_get_path('module', 'legislature') .'/xml.inc';

  $url = 'http://www.govtrack.us/data/us/'. $session .'/bills.amdt/'. $amendment_number .'.xml';
  $result = drupal_http_request($url);
  legislature_govtrack_amendment_xml('setup', NULL, $bill_id);
  if (!legislature_xml_parse($result->data, 'legislature_govtrack_amendment_xml')) {
    watchdog('legislature', t('Failed to update amendment %amendment on bill %id. (%url) %error', array('%amendment' => $amendment_number, '%id' => $bill_id, '%url' => $url, '%error' => $result->error)));
  }
}

function legislature_govtrack_amendment_xml($op, $parser, $data, $attributes = NULL) {
  static $amendment;
  static $last_start;

  switch ($op) {
    case 'setup':
      $amendment = array(
        'bill_id' => $data,
      );

    case 'start':
      switch ($data) {
        case 'AMENDMENT':
          $amendment['location'] = $attributes['CHAMBER'];
          $amendment['number'] = $attributes['NUMBER'];
          break;

        case 'AMENDS':
          $amendment['sequence'] = $attributes['SEQUENCE'];
          break;

        case 'SPONSOR':
          $amendment['sponsor_politician_id'] = legislature_get_politician_id('govtrack', $attributes['ID']);
          break;

        case 'OFFERED':
          $amendment['date'] = $attributes['DATE'];
          break;

        case 'DESCRIPTION':
        case 'PURPOSE':
          $last_start = $data;
          break;
      }
      break;

    case 'data':
      switch ($last_start) {
        case 'DESCRIPTION':
          $amendment['description'] = $data;
          break;

        case 'PURPOSE':
          $amendment['purpose'] = $data;
          break;
      }
      $last_start = NULL;
      break;

    case 'end':
      switch ($data) {
        case 'AMENDMENT':
          legislature_save_amendment($amendment);
          break;
      }
      break;
  }
}

function legislature_update_votes_xml($op, $parser, $data, $attributes = NULL) {
  static $action_id;

  $votes = array(
    '+' => 'AYE',
    '-' => 'NOE',
    '0' => 'NV',
    'P' => 'NV',
  );

  switch ($op) {
    case 'setup':
      $action_id = $data['action_id'];
      db_query('DELETE FROM {legislature_action_politician} WHERE action_id = %d', $action_id);
      break;

    case 'start':
      switch ($data) {
        case 'ROLL':
          $action = array(
            'action_id' => $action_id,
            'ayes' => $attributes['AYE'],
            'noes' => $attributes['NAY'],
          );
          legislature_save_action($action);
          break;

        case 'VOTER':
          $politician_id = legislature_get_politician_id('govtrack', $attributes['ID']);
          if (db_num_rows(db_query('SELECT vote FROM {legislature_action_politician} WHERE politician_id = %d AND action_id = %d', $politician_id, $action_id)) == 0) {
            db_query("INSERT INTO {legislature_action_politician} (politician_id, action_id, vote) VALUES (%d, %d, '%s')", $politician_id, $action_id, $votes[$attributes['VOTE']]);
          }
          break;
      }
      break;
  }
}

function legislature_govtrack_update_bill_votes_xml($op, $parser, $data, $attributes = NULL) {
  static $bill;
  static $vote;
  static $locations = array(
    'h' => 'House',
    's' => 'Senate',
  );
  static $last_start;

  switch ($op) {
    case 'setup':
      $bill = $data;
      break;

    case 'start':
      $last_start = $data;
      switch ($data) {
        case 'VOTE':
          $vote = array(
            'bill_id' => $bill->bill_id,
            'description' => 'vote',
            'location' => $locations[$attributes['CHAMBER']],
            'is_vote' => TRUE,
            'is_substantive' => TRUE,
            'vote_roll' => $attributes['ROLL'],
          );
          break;
      }
      break;

    case 'data':
      switch ($last_start) {
        case 'DESCRIPTION':
          $vote['motion'] = $data;
          break;

        case 'DATE':
          $vote['date_int'] = strtotime($data .' America/New_York');
          $vote['date'] = format_date($vote['date_int'], 'custom', 'Ymd');
          break;

        case 'RESULT':
          if (stristr($data, 'passed') !== FALSE || stristr($data, 'agreed') !== FALSE) {
            $vote['outcome'] = 'PASS';
          }
          else if (stristr($data, 'failed') !== FALSE || stristr($data, 'rejected') !== FALSE) {
            $vote['outcome'] = 'FAIL';
          }
          else {
            watchdog('legislature', t('Vote outcome not found in %result', array('%result' => $data)), WATCHDOG_ERROR);
          }
          break;
      }
      break;

    case 'end':
      switch ($data) {
        case 'VOTE':
          legislature_save_action($vote, array('bill_id', 'vote_roll', 'location'));
          legislature_govtrack_roll($vote['action_id']);
          $vote = NULL;
          break;
      }
      break;
  }
}

function legislature_govtrack_bill_texts() {
  set_time_limit(0);
  $result = db_query("SELECT session_name FROM {legislature_session} WHERE session_jurisdiction = 'us'");
  while ($session = db_fetch_object($result)) {
    $session_directory = opendir(variable_get('legislature_govtrack_bill_text_directory', '') .'/'. $session->session_name);
    while (($directory = readdir($session_directory)) !== FALSE) {
      if ($directory != '.' && $directory != '..') {
        $texts_directory = opendir(variable_get('legislature_govtrack_bill_text_directory', '') .'/'. $session->session_name .'/'. $directory);
        $files = array();
        while (($file = readdir($texts_directory)) !== FALSE) {
          if ($file != '.' && $file != '..') {
            $files[] = $file;
          }
        }
        foreach (preg_grep('/^'. $directory .'\d*.txt$/', $files) as $bill) {
          $bill = str_replace('.txt', '', $bill);
          job_queue_add('legislature_govtrack_bill_text', t('Import bill text for %measure', array('%measure' => $bill)), array($session->session_name, $directory, $bill, preg_grep('/^'. $bill .'[a-z]*\.gen\.html$/', $files)), drupal_get_path('module', 'legislature') .'/imports/us/govtrack.inc');
        }
        closedir($texts_directory);
      }
    }
    closedir($session_directory);
  }
}

function legislature_govtrack_bill_text($session, $directory, $measure, $files) {
  $match = array();

  // Read the files for this bill into an array.
  $texts = array();
  foreach ($files as $file) {
    $type = str_replace(array($measure, '.gen.html'), array('', ''), $file);
    $texts[$type] = file_get_contents(variable_get('legislature_govtrack_bill_text_directory', '') .'/'. $session .'/'. $directory .'/'. $file);
  }

  // Determine the order of the versions.
  $order = array();
  foreach ($texts as $key => $text) {
    if (preg_match('/ previous-status="([a-z]*)" /', $text, $match)) {
      $order[$match[1]] = $key;
    }
    else {
      $order[0] = $key;
    }
  }

  // Save them.
  preg_match('/^([a-z]*)([0-9]*)$/', $measure, $match);
  list($dummy, $prefix, $number) = $match;
  $bill_id = db_result(db_query("SELECT b.bill_id FROM {legislature_bill} b INNER JOIN {legislature_session} s ON s.session_id = b.session_id AND s.session_jurisdiction = 'us' AND s.session_name = '%s' WHERE b.measure = '%s'", $session, drupal_strtoupper($prefix) .' '. $number));
  db_query('DELETE FROM {legislature_bill_text} WHERE bill_id = %d', $bill_id);

  $key = 0;
  $weight = 0;
  while (isset($order[$key])) {
    $key = $order[$key];
    db_query("INSERT INTO {legislature_bill_text} (bill_id, type, weight, text) VALUES (%d, '%s', %d, '%s')", $bill_id, $key, $weight, $texts[$key]);
    $weight += 1;
  }
}

function legislature_govtrack_roll($action_id) {
  include_once drupal_get_path('module', 'legislature') .'/xml.inc';

  $action = db_fetch_object(db_query('SELECT s.session_name, a.location, a.date_int, a.vote_roll, a.bill_id FROM {legislature_action} a INNER JOIN {legislature_bill} b ON b.bill_id = a.bill_id INNER JOIN {legislature_session} s ON s.session_id = b.session_id WHERE a.action_id = %d', $action_id));
  if (empty($action)) {
    watchdog('legislature', t('Failed to update roll vote for action %id, action does not exist.', array('%id' => $action_id)));
    return;
  }
  $response = drupal_http_request('http://www.govtrack.us/data/us/'. $action->session_name .'/rolls/'. drupal_strtolower(substr($action->location, 0, 1)) . format_date($action->date_int, 'custom', 'Y') .'-'. $action->vote_roll .'.xml');
  if (!legislature_xml_parse($response->data, 'legislature_update_votes_xml', array('action_id' => $action_id))) {
    watchdog('legislature', t('Failed to update roll vote for action %id.', array('%id' => $action_id)));
    job_queue_add('legislature_govtrack_roll', t('Update roll vote for action %id (requeued).', array('%id' => $action_id)), array($action_id), drupal_get_path('module', 'legislature') .'/imports/us/govtrack.inc');
  }
}

function legislature_govtrack_committees() {
  include_once drupal_get_path('module', 'legislature') .'/xml.inc';

  foreach (legislature_get_enabled_sessions('us') as $session) {
    $response = drupal_http_request('http://www.govtrack.us/data/us/'. $session .'/committees.xml');
    if (!legislature_xml_parse($response->data, 'legislature_govtrack_committees_xml', array('session' => $session))) {
      watchdog('legislature', t('Failed to update committees for %id.', array('%id' => $session)));
    }
  }
  watchdog('legislature', t('Imported committees from Govtrack'));
}

function legislature_govtrack_committees_xml($op, $parser, $data, $attributes = NULL) {
  static $session_id, $committee_id, $committee_name_session;
  static $sessions = array();

  switch ($op) {
    case 'setup':
      $result = db_query("SELECT session_id, session_name FROM {legislature_session} WHERE session_jurisdiction = 'us'");
      while ($session = db_fetch_object($result)) {
        $sessions[$session->session_name] = $session->session_id;
      }

      $session_id = $sessions[$data['session']];
      break;

    case 'start':
      switch ($data) {
        case 'COMMITTEE':
          if ($attributes['TYPE'] != 'joint') {
            $office_id = legislature_get_office_id('US', drupal_ucfirst($attributes['TYPE']));
            $committee_id = legislature_committee($office_id, $session_id, $attributes['DISPLAYNAME'], $attributes['URL']);
            db_query('DELETE FROM {legislature_committee_membership} WHERE committee_id = %d', $committee_id);
            db_query('DELETE FROM {legislature_committee_name} WHERE committee_id = %d', $committee_id);

            // Found only in 109th.
            if (isset($attributes['THOMASNAME'])) {
              db_query("INSERT INTO {legislature_committee_name} (committee_id, session_id, name) VALUES (%d, %d, '%s')", $committee_id, $session_id, drupal_ucfirst($attributes['TYPE']) .' '. $attributes['THOMASNAME']);
            }
          }
          else {
            $committee_id = NULL;
          }
          break;

        case 'SUBCOMMITTEE':
          $committee_id = NULL;
          break;

        case 'MEMBER':
          if (!is_null($committee_id)) {
            $politician_id = legislature_get_politician_id('govtrack', $attributes['ID']);
            if ($politician_id == 0) {
              watchdog('legislature', t('Could not find politician %politician_id.', array('%politician_id' => $attributes['ID'])), WATCHDOG_WARNING);
            }
            else {
              db_query("INSERT INTO {legislature_committee_membership} (politician_id, committee_id, position) VALUES (%d, %d, '%s')", $politician_id, $committee_id, $attributes['ROLE']);
            }
          }
          break;

        case 'NAME':
          // Under THOMAS-NAMES in 110th only.
          if (!is_null($committee_id)) {
            $committee_name_session = $sessions[$attributes['SESSION']];
          }
          break;
      }
      break;

    case 'data':
      if ($committee_name_session == $session_id) {
        db_query("INSERT INTO {legislature_committee_name} (committee_id, session_id, name) VALUES (%d, %d, '%s')", $committee_id, $committee_name_session, $data);
        $committee_name_session = NULL;
      }
      break;
  }
}

/**
 * Look for new votes in the current session and schedule bill updates.
 */
function legislature_govtrack_recent_votes() {
  include_once drupal_get_path('module', 'legislature') .'/xml.inc';

  $response = drupal_http_request('http://www.govtrack.us/congress/votes_download_xml.xpd');
  if (!legislature_xml_parse($response->data, 'legislature_govtrack_recent_votes_xml')) {
    watchdog('legislature', t('Failed to download recent votes from Govtrack'));
  }
}

function legislature_govtrack_recent_votes_xml($op, $parser, $data, $attributes = NULL) {
  static $houses = array(
    's' => 'Senate',
    'h' => 'House',
  );
  static $done = FALSE;
  static $session = NULL;

  if (!$done) {
    switch ($op) {
      case 'start':
        switch ($data) {
          case 'VOTE':
            $result = db_query("SELECT action_id FROM {legislature_action} WHERE date_int > %d AND location = '%s' AND vote_roll = %d", gmmktime(0, 0, 0, 1, 1, $attributes['YEAR']), $houses[$attributes['CHAMBER']], $attributes['ROLL']);
            if (db_num_rows($result) == 0) {
              if (is_null($session)) {
                $session = max(legislature_get_enabled_sessions('us'));
              }
              // We do not have this vote yet, need to find the bill to update.
              // Unfortunately, that is not stored in a machine-readable format
              // in this file. Download the vote's XML file.
              $response = drupal_http_request('http://www.govtrack.us/data/us/'. $session .'/rolls/'. $attributes['CHAMBER'] . $attributes['YEAR'] .'-'. $attributes['ROLL'] .'.xml');
              if (!legislature_xml_parse($response->data, 'legislature_govtrack_recent_votes_bill_xml', array('session' => $session))) {
                watchdog('legislature', t('Failed to download a recent roll from Govtrack'));
              }
            }
            else {
              // Since votes stored in chonological order, we can assume we have
              // all of the remaining votes.
              $done = TRUE;
            }
            break;
        }
        break;
    }
  }
}

function legislature_govtrack_recent_votes_bill_xml($op, $parser, $data, $attributes = NULL) {
  static $session;
  static $bill_ids = array();

  switch ($op) {
    case 'setup':
      $session = $data['session'];
      break;

    case 'start':
      switch ($data) {
        case 'BILL':
          $result = db_query("SELECT b.bill_id FROM {legislature_bill} b INNER JOIN {legislature_session} s ON s.session_id = b.session_id WHERE s.session_name = '%s' AND s.session_jurisdiction = 'us' AND measure = '%s'", $session, drupal_strtoupper($attributes['TYPE'] .' '. $attributes['NUMBER']));
          if (db_num_rows($result) == 1) {
            $bill = db_fetch_object($result);
            if (!in_array($bill->bill_id, $bill_ids)) {
              job_queue_add('legislature_govtrack_update_bill', t('Download bill %id', array('%id' => $bill->bill_id)), array($bill->bill_id), drupal_get_path('module', 'legislature') .'/imports/us/govtrack.inc');
              // Avoid adding the same bill twice.
              $bill_ids[] = $bill->bill_id;
            }
          }
          break;
      }
      break;
  }
}

/**
 * Find undervotes, senate and house with less than 100 and 435 respectively.
 */
function legislature_govtrack_undervotes() {
  $bill_ids = array();

  foreach (array('House' => 100, 'Senate' => 435) as $location => $count) {
    $result = db_query("SELECT a.bill_id, a.location, count(*) c FROM {legislature_action} a INNER JOIN {legislature_bill} b ON b.bill_id = a.bill_id INNER JOIN {legislature_session} s ON s.session_id = b.session_id AND s.session_jurisdiction = 'us' LEFT JOIN {legislature_action_politician} ap ON ap.action_id = a.action_id WHERE a.location = '%s' GROUP BY a.action_id HAVING c != %d", $location, $count);
    while ($action = db_fetch_object($result)) {
      $bill_ids[$action->bill_id] = TRUE;
    }
  }

  foreach ($bill_ids as $bill_id => $dummy) {
    job_queue_add('legislature_govtrack_update_bill', t('Download bill %id', array('%id' => $bill_id)), array($bill_id), drupal_get_path('module', 'legislature') .'/imports/us/govtrack.inc');
  }
}
